package PbT::UserData;

use strict;

use base 'PbT::GEN::UserData';

use Gantry::Plugins::AutoCRUD qw(
    do_add
    do_edit
    do_delete
    form_name
    write_file
);

use PbT::Model::user_data qw(
    $USER_DATA
);
use PbT::Model;
sub schema_base_class { return 'PbT::Model'; }
use Gantry::Plugins::DBIxClassConn qw( get_schema );

#-----------------------------------------------------------------
# $self->do_main(  )
#-----------------------------------------------------------------
# This method supplied by PbT::GEN::UserData

#-----------------------------------------------------------------
# $self->form( $row )
#-----------------------------------------------------------------
# This method supplied by PbT::GEN::UserData


#-----------------------------------------------------------------
# get_model_name( )
#-----------------------------------------------------------------
sub get_model_name {
    return $USER_DATA;
}

#-----------------------------------------------------------------
# get_orm_helper( )
#-----------------------------------------------------------------
sub get_orm_helper {
    return 'Gantry::Plugins::AutoCRUDHelper::DBIxClass';
}

#-----------------------------------------------------------------
# text_descr( )
#-----------------------------------------------------------------
sub text_descr     {
    return 'user';
}

1;

=head1 NAME

PbT::UserData - A controller in the PbT application

=head1 SYNOPSIS

This package is meant to be used in a stand alone server/CGI script or the
Perl block of an httpd.conf file.

Stand Alone Server or CGI script:

    use PbT::UserData;

    my $cgi = Gantry::Engine::CGI->new( {
        config => {
            #...
        },
        locations => {
            '/someurl' => 'PbT::UserData',
            #...
        },
    } );

httpd.conf:

    <Perl>
        # ...
        use PbT::UserData;
    </Perl>

    <Location /someurl>
        SetHandler  perl-script
        PerlHandler PbT::UserData
    </Location>

If all went well, one of these was correctly written during app generation.

=head1 DESCRIPTION

This module is the admin interface to the user_data table (not exposed
to the public).

=head1 METHODS

=over 4

=item get_model_name

=item text_descr

=item schema_base_class

=item get_orm_helper


=back


=head1 METHODS INHERITED FROM PbT::GEN::UserData

=over 4

=item do_main

=item form


=back


=head1 DEPENDENCIES

    PbT
    PbT::GEN::UserData
    PbT::Model::user_data
    Gantry::Plugins::AutoCRUD

=head1 AUTHOR

Phil Crow, E<lt>phil@localdomainE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2007 Phil Crow

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.6 or,
at your option, any later version of Perl 5 you may have available.

=cut
